unit PalmDM;

{$B-}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  TabdPalmConduit, TabdPalmFields;

{ TODO : Create a conduit entry using CondCfg.exe

  This demo assumes that there is a conduit entry created with :
      Type       = Application
      Conduit    = DelphiConduit.dll
      Creator Id = memo  (you should use the creator id of an application on
                          your Palm that is not already used by another conduit)

  Use the CondCfg.exe utility that comes with the CDK to create the new entry.

  Got to Project\Options... and then the Directories/Conditional tab and
  check that the Output Directory is the Palm directory (whic is normally
  c:\Program Files\Palm by default).
}


type
  TPalmData = class(TDataModule)
    Conduit: TTabdConduit;
    PalmTodoDB: TTabdConduitDb;
    PalmDatebookDB: TTabdConduitDb;
    PalmAddressDB: TTabdConduitDb;
    PalmMemoDB: TTabdConduitDb;
    procedure ConduitOpenConduit(var Error: Integer);
    procedure PalmDatebookDBTestFieldExistsForRead(Sender: TTabdConduitDb;
      Field: TtabdPalmField; ArrayIndex: Integer;
      var FieldExists: Boolean);
    procedure PalmDatebookDBTestFieldExistsForWrite(Sender: TTabdConduitDb;
      Field: TtabdPalmField; ArrayIndex: Integer;
      var FieldExists: Boolean);
    procedure PalmAddressDBTestFieldExistsForRead(Sender: TTabdConduitDb;
      Field: TtabdPalmField; ArrayIndex: Integer;
      var FieldExists: Boolean);
    procedure PalmAddressDBTestFieldExistsForWrite(Sender: TTabdConduitDb;
      Field: TtabdPalmField; ArrayIndex: Integer;
      var FieldExists: Boolean);
  private
    procedure SyncAddressDb;
    procedure SyncDatebookDb;
    procedure SyncMemoDb;
    procedure SyncTodoDb;
    procedure TraceFields(TheFields : TTabdPalmFields;const DebugStr : string);
    function  GetPhoneName(PhoneField : Integer) : string;  // Expects 1 to 5
    procedure SetPhoneValue(var Options : Longint;const PhoneType : string;
                            PhoneField : Integer);
  public
  end;

var
  PalmData: TPalmData;

implementation

uses
  Math;

{$R *.DFM}

const
  PhoneNames : array[0..7] of string = ('Work','Home','Fax','Other','EMail',
                                        'Main','Pager','Mobile');


procedure TPalmData.ConduitOpenConduit(var Error: Integer);
begin
  Error := 0;
  SyncAddressDb;
//  SyncDatebookDb;
//  SyncMemoDb;
//  SyncTodoDb;
end;


procedure TPalmData.SyncAddressDb;
begin
  PalmAddressDB.Open;
  PalmAddressDB.ReadAppInfo;
  TraceFields(PalmAddressDB.AppInfoFields,PalmAddressDB.DebugAppInfoAsStr);

  try
    // Read all the existing records
    while not PalmAddressDB.Eof do begin
      TraceFields(PalmAddressDB.Fields,PalmAddressDB.DebugRawRecAsStr);
      PalmAddressDB.Next;
      Conduit.YieldCycles(1);
    end;

    // Normally you would do this
//    PalmAddressDB.PurgeDeletedRecords;
//    PalmAddressDB.ResetSyncFlags;

    // Add a new Address record
{ TODO : Remove this comment to enable writing to the Address book
    PalmAddressDB.Append;
    PalmAddressDB.FieldByName('Name').AsString := 'Address added at ' +
                                                  DateTimeToStr(Now);
    PalmAddressDB.FieldByName('Company').AsString := 'Conduit Test Company';
    PalmAddressDB.Post;
TODO : Remove this comment to enable writing to the Address book }
  finally
    PalmAddressDB.Close;
  end;
end;


procedure TPalmData.SyncDatebookDb;

  function AnHourFromNow : TDateTime;
    var
      Hr,Min,Sec,mSec : Word;
  begin
    DecodeTime(Time,Hr,Min,Sec,mSec);
    Result := EncodeTime(Hr + 1,Min,Sec,mSec);
  end;

begin
  PalmDatebookDB.Open;
  PalmDatebookDB.ReadAppInfo;
  TraceFields(PalmDatebookDB.AppInfoFields,PalmDatebookDB.DebugAppInfoAsStr);

  try
    // Read all the existing records
    while not PalmDatebookDB.Eof do begin
      TraceFields(PalmDatebookDB.Fields,PalmDatebookDB.DebugRawRecAsStr);
      PalmDatebookDB.Next;
      Conduit.YieldCycles(1);
    end;

    // Normally you would do this
//    PalmDatebookDB.PurgeDeletedRecords;
//    PalmDatebookDB.ResetSyncFlags;

    // Add a new Datebook record
{ TODO : Remove this comment to enable writing to the Datebook
    PalmDatebookDB.Append;
    PalmDatebookDB.FieldByName('StartTime').AsDateTime := Time;
    PalmDatebookDB.FieldByName('EndTime').AsDateTime   := AnHourFromNow;
    PalmDatebookDB.FieldByName('Date').AsDateTime      := Date;
    PalmDatebookDB.FieldByName('Description').AsString := 'Datebook added at ' +
                                                          DateTimeToStr(Now);
    PalmDatebookDB.FieldByName('Note').AsString        := 'The note goes here';
    PalmDatebookDB.Post;
TODO : Remove this comment to enable writing to the Datebook }
  finally
    PalmDatebookDB.Close;
  end;
end;


procedure TPalmData.SyncMemoDb;
begin
  PalmMemoDB.Open;
  PalmMemoDB.ReadAppInfo;
  TraceFields(PalmMemoDB.AppInfoFields,PalmMemoDB.DebugAppInfoAsStr);

  try
    // Read all the existing records
    while not PalmMemoDB.Eof do begin
      TraceFields(PalmMemoDB.Fields,PalmMemoDB.DebugRawRecAsStr);
      PalmMemoDB.Next;
      Conduit.YieldCycles(1);
    end;

    // Normally you would do this
//    PalmMemoDB.PurgeDeletedRecords;
//    PalmMemoDB.ResetSyncFlags;

    // Add a new Memo record
{ TODO : Remove this comment to enable writing to the Memo pad
    PalmMemoDB.Append;
    PalmMemoDB.FieldByName('Note').AsString := 'Memo added at ' +
                                               DateTimeToStr(Now);
    PalmMemoDB.Post;
TODO : Remove this comment to enable writing to the Memo pad }
  finally
    PalmMemoDB.Close;
  end;
end;


procedure TPalmData.SyncTodoDb;
begin
  PalmToDoDB.Open;
  PalmToDoDB.ReadAppInfo;
  TraceFields(PalmToDoDB.AppInfoFields,PalmToDoDB.DebugAppInfoAsStr);

  try
    // Read all the existing records
    while not PalmTodoDB.Eof do begin
      TraceFields(PalmToDoDB.Fields,PalmToDoDB.DebugRawRecAsStr);
      PalmTodoDB.Next;
      Conduit.YieldCycles(1);
    end;

    // Normally you would do this
//    PalmTodoDB.PurgeDeletedRecords;
//    PalmTodoDB.ResetSyncFlags;

    // Add a new Todo record
{ TODO : Remove this comment to enable writing to the Todo list
    PalmTodoDB.Append;
    PalmTodoDB.FieldByName('DueDate').AsDateTime   := Date;  // Can be -1
    PalmTodoDB.FieldByName('Completed').AsInteger  := 1;
    PalmTodoDB.FieldByName('Description').AsString := 'Todo added at ' +
                                                      DateTimeToStr(Now);
    PalmTodoDB.FieldByName('Notes').AsString       := 'The notes go here if required';
    PalmTodoDB.Post;
TODO : Remove this comment to enable writing to the Todo list }
  finally
    PalmTodoDB.Close;
  end;
end;


procedure TPalmData.TraceFields(TheFields : TTabdPalmFields;const DebugStr : string);
  var
    s   : string;
    i,j : Integer;

  function FormatTrace(FieldIndex,ArrayIndex : Integer) : string;
  begin
    case TheFields[FieldIndex].DataType of
      pftDate : Result := DateToStr(TheFields[FieldIndex].AsDateTime);
      pftTime : Result := TimeToStr(TheFields[FieldIndex].AsDateTime);
      pftBlob : Result := StringOfChar('#',TheFields[FieldIndex].Size);
      else
        Result := VarToStr(TheFields[FieldIndex].ArrayValue[ArrayIndex]);
    end;
  end;

begin
  s := '';

  for i := 0 to TheFields.Count - 1 do begin
    if TheFields[i].ArraySize = 1 then begin
      s := s + TheFields[i].FieldName + ' = ' + FormatTrace(i,0) + #13#10;
    end else if TheFields[i].ArraySize > 1 then begin
      for j := 0 to TheFields[i].ArraySize - 1 do begin
        s := s + TheFields[i].FieldName + '[' + IntToStr(j) + '] = ' +
             FormatTrace(i,j) + #13#10;
      end;
    end;
  end;

  s := s + '------------------------------' + #13#10;
  // Use this method to see what the database actually looks like if you
  // need to do a bit of reverse engineering.
  s := s + DebugStr;

  ShowMessage(s);
end;


function TPalmData.GetPhoneName(PhoneField : Integer) : string;  // Expects 1 to 5
  var
    TempLong : Longint;
    Index    : Integer;
begin
  // Result will default to 'Other' if there is a problem
  TempLong := PalmAddressDb.FieldByName('Options').Value;
  Index    := StrToIntDef(Copy(Format('%8.8x',[TempLong]),9 - PhoneField,1),3);

  if (Index < Low(PhoneNames)) or (Index > High(PhoneNames)) then begin
    // Just in case...
    Index := 3;
  end;

  Result := PhoneNames[Index];
end;


procedure TPalmData.SetPhoneValue(var Options : Longint;const PhoneType : string;
                                  PhoneField : Integer);
  var
    i    : Integer;
    Temp : string;
begin
  for i := Low(PhoneNames) to High(PhoneNames) do begin
    if CompareText(PhoneNames[i],PhoneType) = 0 then begin
      Break;
    end;
  end;

  Temp                 := Format('%8.8x',[Options]);
  Temp[9 - PhoneField] := IntToStr(i)[1];
  Options              := StrToInt('$' + Temp);
end;


procedure TPalmData.PalmDatebookDBTestFieldExistsForRead(
            Sender: TTabdConduitDb; Field: TtabdPalmField; ArrayIndex: Integer;
            var FieldExists: Boolean);
  var
    FlagValue : Byte;
begin
  FlagValue := PalmDatebookDB.FieldByName('Flags').AsInteger;

  // The field order doesn't correspond to the flag bits!!
  case Field.Index of
    0..4  : FieldExists := True;  // All fields down to Filler1
    5,6   : FieldExists := (FlagValue and 64) > 0;  // AlarmAdvanceXXX fields
    7..12 : FieldExists := (FlagValue and 32) > 0;  // RepeatXXX fields and Filler2
    13    : FieldExists := (FlagValue and 8) > 0;   // NumOfExceptions
    14    : FieldExists := ((FlagValue and 8) > 0) and    // Exceptions array
                           (PalmDatebookDB.FieldByName('NumOfExceptions').AsInteger > ArrayIndex);
    15    : FieldExists := (FlagValue and 4) > 0;   // Description
    16    : FieldExists := (FlagValue and 16) > 0;  // Note
  end;
end;


procedure TPalmData.PalmDatebookDBTestFieldExistsForWrite(
            Sender: TTabdConduitDb; Field: TtabdPalmField; ArrayIndex: Integer;
            var FieldExists: Boolean);
  var
    FlagValue : Byte;
begin
  if Field.Index <= 4 then begin
    FieldExists := True;
    Exit;
  end;

  FlagValue   := PalmDatebookDB.FieldByName('Flags').AsInteger;
  FieldExists := not VarIsEmpty(Field.ArrayValue[ArrayIndex]);

  if not FieldExists then begin
    Exit;
  end;

  // The field order doesn't correspond to the flag bits!!
  case Field.Index of
    5,6   : FlagValue := FlagValue or 64;  // AlarmAdvanceXXX fields
    7..12 : FlagValue := FlagValue or 32;  // RepeatXXX fields and Filler2
    13    : FlagValue := FlagValue or 8;   // NumOfExceptions
    14    : FieldExists := ((FlagValue and 8) > 0) and    // Exceptions array
                           (PalmDatebookDB.FieldByName('NumOfExceptions').AsInteger > ArrayIndex);
    15    : FlagValue := FlagValue or 4;   // Description
    16    : FlagValue := FlagValue or 16;  // Note
  end;

  PalmDatebookDB.FieldByName('Flags').AsInteger := FlagValue;
end;


procedure TPalmData.PalmAddressDBTestFieldExistsForRead(
            Sender: TTabdConduitDb; Field: TtabdPalmField; ArrayIndex: Integer;
            var FieldExists: Boolean);
  var
    PowerOfTwo : Longint;
begin
  if Field.Index <= 2 then begin
    FieldExists := True;
    Exit;
  end;

  PowerOfTwo  := Trunc(IntPower(2,Field.Index - 3));
  FieldExists := (PalmAddressDb.FieldByName('Flags').AsInteger and PowerOfTwo) > 0;
end;


procedure TPalmData.PalmAddressDBTestFieldExistsForWrite(
            Sender: TTabdConduitDb; Field: TtabdPalmField; ArrayIndex: Integer;
            var FieldExists: Boolean);
  var
    PowerOfTwo : Longint;
begin
  if Field.Index <= 2 then begin
    FieldExists := True;
    Exit;
  end;

  FieldExists := not VarIsEmpty(Field.Value);

  if FieldExists then begin
    PowerOfTwo := Trunc(IntPower(2,Field.Index - 3));
    PalmAddressDb.FieldByName('Flags').AsInteger :=
                     PalmAddressDb.FieldByName('Flags').AsInteger or PowerOfTwo;
  end;
end;


end.
